@@ -1,7 +1,7 @@
 // Filename: emulator.rs
 // Author:	 Kai Rese
-// Version:	 0.3
-// Date:	 12-07-2021 (DD-MM-YYYY)
+// Version:	 0.4
+// Date:	 14-11-2021 (DD-MM-YYYY)
 // Library:  gpcas_arschitek_zero
 //
 // Copyright (c) 2021 Kai Rese
@@ -31,10 +31,8 @@ use std::sync::atomic::{AtomicUsize, Ordering};
 pub struct ArschitekZeroEmulator {
     /// The current instruction pointer value.
     ip: usize,
-    /// The count of emulation calls.
-    decoded_instruction_count: AtomicUsize,
-    /// The count of emulated and retired instructions.
-    retired_instruction_count: AtomicUsize,
+    /// The count of emulated instructions.
+    instruction_count: AtomicUsize,
     /// If the emulation is still running or has been terminated.
     running: bool,
     /// The memory data of the program image.
@@ -54,8 +52,7 @@ impl ArschitekZeroEmulator {
         registers[14] = u16::MAX;
         ArschitekZeroEmulator {
             ip: 0,
-            decoded_instruction_count: AtomicUsize::new(0),
-            retired_instruction_count: AtomicUsize::new(0),
+            instruction_count: AtomicUsize::new(0),
             running: true,
             memory: executable_data,
             registers,
@@ -66,7 +63,7 @@ impl ArschitekZeroEmulator {
 
 impl Emulator for ArschitekZeroEmulator {
     #[inline]
-    fn emulate_instruction(&mut self, address: usize) -> Option<Instruction> {
+    fn emulate_instruction(&mut self, address: usize, commit: bool) -> Option<Instruction> {
         if self.running {
             let raw_instruction_data =
                 u16::from_le_bytes(self.memory[address..address + 2].try_into().unwrap());
@@ -86,8 +83,7 @@ impl Emulator for ArschitekZeroEmulator {
                 size: 2,
             };
 
-            self.decoded_instruction_count
-                .fetch_add(1, Ordering::Relaxed);
+            self.instruction_count.fetch_add(1, Ordering::Relaxed);
             // is hard wired, gets reset so output checks can be omitted
             self.registers[15] = 0;
 
@@ -98,9 +94,7 @@ impl Emulator for ArschitekZeroEmulator {
                         .wrapping_add(2)
                         .wrapping_add(template_data.imm8 << 1)
                         as usize;
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.ip += 2;
                         self.registers[template_data.reg0 as usize] = u16::from_le_bytes(
                             self.memory[instruction.memory_address..instruction.memory_address + 2]
@@ -116,9 +110,7 @@ impl Emulator for ArschitekZeroEmulator {
                         .wrapping_add(2)
                         .wrapping_add(template_data.imm8 << 1)
                         as usize;
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.ip += 2;
                         self.memory[instruction.memory_address..instruction.memory_address + 2]
                             .copy_from_slice(
@@ -132,9 +124,7 @@ impl Emulator for ArschitekZeroEmulator {
                         .wrapping_add(2)
                         .wrapping_add(template_data.imm12 << 1)
                         as usize;
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.ip = instruction.branch_address;
                     }
                 }
@@ -153,9 +143,7 @@ impl Emulator for ArschitekZeroEmulator {
                         branch = false;
                     }
 
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         if branch {
                             self.ip = instruction.branch_address;
                         } else {
@@ -170,9 +158,7 @@ impl Emulator for ArschitekZeroEmulator {
                         .wrapping_add(sign_extend_u4(template_data.reg2_imm4))
                         as usize)
                         << 1;
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.ip += 2;
                         self.registers[template_data.reg0 as usize] = u16::from_le_bytes(
                             self.memory[instruction.memory_address..instruction.memory_address + 2]
@@ -189,9 +175,7 @@ impl Emulator for ArschitekZeroEmulator {
                         .wrapping_add(sign_extend_u4(template_data.reg2_imm4) << 1)
                         as usize)
                         << 1;
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.ip += 2;
                         self.memory[instruction.memory_address..instruction.memory_address + 2]
                             .copy_from_slice(
@@ -206,9 +190,7 @@ impl Emulator for ArschitekZeroEmulator {
                         .wrapping_add(template_data.imm8)
                         as usize)
                         << 1;
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.ip = instruction.branch_address;
                     }
                 }
@@ -227,9 +209,7 @@ impl Emulator for ArschitekZeroEmulator {
                         branch = false;
                     }
 
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         if branch {
                             self.ip = instruction.branch_address;
                         } else {
@@ -241,9 +221,7 @@ impl Emulator for ArschitekZeroEmulator {
                 0x8 => {
                     instruction.inputs[0] = template_data.reg1;
                     instruction.inputs[1] = template_data.reg2_imm4;
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.ip += 2;
                         self.registers[template_data.reg0 as usize] = self.registers
                             [template_data.reg1 as usize]
@@ -253,9 +231,7 @@ impl Emulator for ArschitekZeroEmulator {
                 // Add immediate
                 0x9 => {
                     instruction.inputs[0] = template_data.reg0;
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.ip += 2;
                         self.registers[template_data.reg0 as usize] = self.registers
                             [template_data.reg0 as usize]
@@ -266,9 +242,7 @@ impl Emulator for ArschitekZeroEmulator {
                 0xa => {
                     instruction.inputs[0] = template_data.reg1;
                     instruction.inputs[1] = template_data.reg2_imm4;
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.ip += 2;
                         self.registers[template_data.reg0 as usize] = self.registers
                             [template_data.reg1 as usize]
@@ -277,9 +251,7 @@ impl Emulator for ArschitekZeroEmulator {
                 }
                 // Set to target
                 0xb => {
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.ip += 2;
                         self.registers[template_data.reg0 as usize] =
                             (self.ip as u16 >> 1).wrapping_add(template_data.imm8);
@@ -288,9 +260,7 @@ impl Emulator for ArschitekZeroEmulator {
                 // Shift left
                 0xc => {
                     instruction.inputs[0] = template_data.reg1;
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.ip += 2;
                         self.registers[template_data.reg0 as usize] = self.registers
                             [template_data.reg1 as usize]
@@ -300,9 +270,7 @@ impl Emulator for ArschitekZeroEmulator {
                 // Shift right
                 0xd => {
                     instruction.inputs[0] = template_data.reg1;
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.ip += 2;
                         self.registers[template_data.reg0 as usize] = self.registers
                             [template_data.reg1 as usize]
@@ -313,9 +281,7 @@ impl Emulator for ArschitekZeroEmulator {
                 0xe => {
                     instruction.inputs[0] = template_data.reg0;
                     instruction.inputs[1] = template_data.reg1;
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.ip += 2;
                         let first_operand = self.registers[template_data.reg0 as usize];
                         let second_operand = self.registers[template_data.reg1 as usize];
@@ -335,9 +301,7 @@ impl Emulator for ArschitekZeroEmulator {
                 }
                 // Stop execution
                 0xf => {
-                    if address == self.ip {
-                        self.retired_instruction_count
-                            .fetch_add(1, Ordering::Relaxed);
+                    if commit {
                         self.running = false;
                         writeln!(
                             self.output_buffer,
@@ -369,13 +333,8 @@ impl Emulator for ArschitekZeroEmulator {
     }
 
     #[inline]
-    fn get_decoded_instruction_count(&self) -> usize {
-        self.decoded_instruction_count.load(Ordering::Relaxed)
-    }
-
-    #[inline]
-    fn get_retired_instruction_count(&self) -> usize {
-        self.retired_instruction_count.load(Ordering::Relaxed)
+    fn get_instruction_count(&self) -> usize {
+        self.instruction_count.load(Ordering::Relaxed)
     }
 
     #[inline]
